<template>
  <div class="mx-2 my-3">
    <div class="row bg-gray-100 border rounded py-2">
      <div class="col-8 text-start p-0 d-flex flex-row" ref="dropdown">
        <div class="drag-handle d-flex cursor-move">
          <span class="m-auto mx-1 px-2">☰</span>
        </div>
        <div class="w-100">
          <input
            class="w-100 h4 m-0 ps-2 rounded form-control"
            :class="{'is-invalid': task.name === ''}"
            :id="generateInputId(activityIndex, taskIndex, 'taskName')"
            :aria-describedby="generateInputId(activityIndex, taskIndex, 'taskNameFeedback')"
            required
            placeholder="Task Name"
            type="text"
            v-model="task.name"
            @input="filterTaskOptions"
            @keydown.down="onTaskInputArrowDown"
            @keydown.up="onTaskInputArrowUp"
            @keydown.enter="selectTaskOption"
            @focus="showDropdownTaskOptions" />
          <ul
            :id="`${task.name}-dropdown`"
            class="dropdown-menu d-block max-h-300px overflow-auto p-0 m-0"
            v-if="taskFilteredOptions.length > 0 && showTaskOptions">
            <li v-for="(taskOption, index) in taskFilteredOptions"
              :key="`${task.name}-${index}`"
              class="py-1 px-2 cursor-pointer"
              :class="{ 'active': index === taskActiveIndex }"
              @mousedown="selectTaskOption(index)"
              @mouseenter="taskActiveIndex = index">
              {{ taskOption.name }}
            </li>
          </ul>
          <div
            :id="generateInputId(activityIndex, taskIndex, 'taskNameFeedback')"
            class="invalid-feedback"
            v-if="task.name === ''"
          >
            Please provide a task name.
          </div>
        </div>
      </div>
      <div class="col-4 d-flex align-items-center justify-content-end pe-1 pe-sm-2">
        <button
          :id="generateInputId(activityIndex, taskIndex, 'taskTrashIcon')"
          class="me-3 btn-unstyled"
          type="button"
          @click="deleteTask(activity, taskIndex)">
          <i class="fa fa-trash" aria-hidden="true"></i>
        </button>
        <button
          class="btn-unstyled"
          @click="() => {task._props.isVisible = !task._props.isVisible}"
        >
          <i v-if="task._props.isVisible" class="fa fa-chevron-up" aria-hidden="true"></i>
          <i v-if="!task._props.isVisible" class="fa fa-chevron-down" aria-hidden="true"></i>
        </button>
      </div>
    </div>
    <div
      v-if="task._props.isVisible"
      class="row bg-white border rounded-bottom p-2 mt-n2px pb-4"
    >
      <div class="col-md-6 mb-3 mb-md-0">
        <div class="card">
          <div class="card-header text-center">
            <h5 class="m-0">Risk(s)</h5>
          </div>
          <div class="card-body text-center">
            <ul class="list-unstyled">
              <li v-for="(risk, riskIndex) in task.risks">
                <div class="d-flex flex-column mt-1">
                  <div class="w-100 d-flex flex-row">
                    <textarea
                      class="w-100 form-control"
                      :class="{'is-invalid': risk === '' && getStringListFiltered(task.risks).length === 0}"
                      :id="generateInputId(activityIndex, taskIndex, `taskRisk${riskIndex}`)"
                      :aria-describedby="generateInputId(activityIndex, taskIndex, `taskRiskFeedback${riskIndex}`)"
                      placeholder="Type Associated Risk"
                      required
                      :value="risk"
                      @input="event => handleTaskRiskValueInputChange(activityIndex, taskIndex, riskIndex, event.target.value)"></textarea>
                    <button
                      class="p-0 ms-2 btn-unstyled"
                      type="button"
                      @click="deleteTaskRisk(task, riskIndex)">
                      X
                    </button>
                  </div>
                  <div
                    :id="generateInputId(activityIndex, taskIndex, `taskRiskFeedback${riskIndex}`)"
                    class="invalid-feedback text-start d-block"
                    v-if="risk === '' && getStringListFiltered(task.risks).length === 0"
                  >
                    Please enter a risk.
                  </div>
                </div>
              </li>
            </ul>
            <button
              class="mt-2 btn-link"
              type="button"
              @click="addTaskRisk(task)">
              <i class="fa fa-plus" aria-hidden="true"></i> Add risk
            </button>
            <div class="invalid-feedback d-block" v-if="task.risks.length === 0">
              Please enter a risk.
            </div>
          </div>
        </div>
      </div>
      <div class="col-md-6">
        <div class="card">
          <div class="card-header text-center">
            <h5 class="m-0">Countermeasure</h5>
          </div>
          <div class="card-body text-center">
            <ul class="list-unstyled">
              <li v-for="(countermeasure, countermeasureIndex) in task.countermeasures">
                <div class="d-flex flex-column mt-1">
                  <div class="w-100 d-flex flex-row">
                    <textarea
                      class="w-100 form-control"
                      :class="{'is-invalid': countermeasure === '' && getStringListFiltered(task.countermeasures).length === 0}"
                      :id="generateInputId(activityIndex, taskIndex, `taskCountermeasure${countermeasureIndex}`)"
                      :aria-describedby="generateInputId(activityIndex, taskIndex, `taskCountermeasureFeedback${countermeasureIndex}`)"
                      placeholder="Type Associated Countermeasure"
                      required
                      :value="countermeasure"
                      @input="event => handleTaskCountermeasureValueInputChange(activityIndex, taskIndex, countermeasureIndex, event.target.value)"></textarea>
                    <button
                      class="p-0 ms-2 btn-unstyled"
                      type="button"
                      @click="deleteTaskCountermeasure(task, countermeasureIndex)">
                      X
                    </button>
                  </div>
                  <div
                    :id="generateInputId(activityIndex, taskIndex, `taskCountermeasureFeedback${countermeasureIndex}`)"
                    class="invalid-feedback text-start d-block"
                    v-if="countermeasure === '' && getStringListFiltered(task.countermeasures).length === 0"
                  >
                    Please enter a countermeasure.
                  </div>
                </div>
              </li>
            </ul>
            <button
              class="mt-2 btn-link"
              type="button"
              @click="addTaskCountermeasure(task)">
              <i class="fa fa-plus" aria-hidden="true"></i> Add countermeasure
            </button>
            <div class="invalid-feedback d-block" v-if="task.countermeasures.length === 0">
              Please enter a countermeasure.
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
</template>

<script>
export default {
  name: 'JobHazardAnalysisCard',
  props: {
    'activity': Object,
    'activityIndex': Number,
    'task': Object,
    'taskIndex': Number,
    'activities': Array,
    'independentTasks': Array,
    'generateInputIdCallback': {
      type: Function,
      required: true,
    },
    'getStringListFilteredCallback': {
      type: Function,
      required: true,
    },
    'taskOptions': {
      type: Array,
      default: [],
    },
  },
  data() {
    return {
      taskFilteredOptions: [],
      taskActiveIndex: -1,
      showTaskOptions: false,
    }
  },
  methods: {
    filterTaskOptions() {
      const regex = new RegExp(this.task.name, 'i');
      this.taskFilteredOptions = this.taskOptions.filter(task => regex.test(task.name));
      this.showTaskOptions = this.taskFilteredOptions.length > 0;
      this.taskActiveIndex = -1; // Reset active index when filtering
      this.handleTaskNameInputChange(this.activityIndex, this.taskIndex, this.task.name);
    },
    onTaskInputArrowDown(event) {
      if (this.taskActiveIndex < this.taskFilteredOptions.length - 1) {
        this.taskActiveIndex++;
      }
    },
    onTaskInputArrowUp(event) {
      if (this.taskActiveIndex > 0) {
        this.taskActiveIndex--;
      }
    },
    selectTaskOption(index) {
      if (this.taskActiveIndex !== -1) {
        const taskPicked = this.taskFilteredOptions[this.taskActiveIndex];

        // Set current task with data from task selected
        if (this.activity !== undefined) {
          this.activity.tasks[this.taskIndex].name = taskPicked.name;
          this.activity.tasks[this.taskIndex].risks = taskPicked.risks;
          this.activity.tasks[this.taskIndex].countermeasures = taskPicked.countermeasures;
        }
        else {
          this.independentTasks[this.taskIndex].name = taskPicked.name;
          this.independentTasks[this.taskIndex].risks = taskPicked.risks;
          this.independentTasks[this.taskIndex].countermeasures = taskPicked.countermeasures;
        }

        // Open collapsed row if it's closed
        this.task._props.isVisible = true;

        this.taskFilteredOptions = [];
        this.showTaskOptions = false;
      }
    },
    showDropdownTaskOptions() {
      this.showTaskOptions = this.taskFilteredOptions.length > 0;
    },
    handleClickOutside(event) {
      if (!this.$refs.dropdown.contains(event.target)) {
        this.showTaskOptions = false;
      }
    },
    generateInputId(activityIndex, taskIndex, inputName) {
      if (typeof this.generateInputIdCallback === 'function') {
        return this.generateInputIdCallback(activityIndex, taskIndex, inputName);
      }
    },
    getStringListFiltered(stringList) {
      if (typeof this.getStringListFilteredCallback === 'function') {
        return this.getStringListFilteredCallback(stringList);
      }
    },
    deleteTask(activity, taskIndex) {
      if (activity !== undefined) {
        activity.tasks.splice(taskIndex, 1);
      }
      else {
        this.independentTasks.splice(taskIndex, 1);
      }
    },
    addTaskRisk(task) {
      task.risks.push('');
    },
    deleteTaskRisk(task, riskIndex) {
      task.risks.splice(riskIndex, 1);
    },
    addTaskCountermeasure(task) {
      task.countermeasures.push('');
    },
    deleteTaskCountermeasure(task, countermeasureIndex) {
      task.countermeasures.splice(countermeasureIndex, 1);
    },
    handleTaskNameInputChange(activityIndex, taskIndex, taskName) {
      if (activityIndex !== undefined) {
        this.activities[activityIndex].tasks[taskIndex].name = taskName;
      }
      else {
        this.independentTasks[taskIndex].name = taskName;
      }
    },
    handleTaskRiskValueInputChange(activityIndex, taskIndex, riskIndex, riskValue) {
      if (activityIndex !== undefined) {
        this.activities[activityIndex].tasks[taskIndex].risks[riskIndex] = riskValue;
      }
      else {  // Independent task
        this.independentTasks[taskIndex].risks[riskIndex] = riskValue;
      }
    },
    handleTaskCountermeasureValueInputChange(activityIndex, taskIndex, countermeasureIndex, countermeasureValue) {
      if (activityIndex !== undefined) {
        this.activities[activityIndex].tasks[taskIndex].countermeasures[countermeasureIndex] = countermeasureValue;
      }
      else {  // Independent task
        this.independentTasks[taskIndex].countermeasures[countermeasureIndex] = countermeasureValue;
      }
    },
  },
  mounted() {
    document.addEventListener('click', this.handleClickOutside);
  },
  beforeUnmount() {
    document.removeEventListener('click', this.handleClickOutside);
  },
};
</script>

<style scoped>
.dropdown-menu li.active {
  background-color: #f0f0f0;
}
</style>
